import ajax from '../util/ajax'
import api from '../util/api'

export default {
  namespaced: true,
  state: {
    activity: {},
    user: '',
    detail: {},
    myprofile: [],
    toplist: [],
    list: [],
    theme: 1,
    code: '',
  },
  mutations: {
    updateActivity(state, data) {
      state.activity = data || {}
      state.theme = data.ActivityId || 1
    },
    updateUser(state, data) {
      state.user = data.openid
      state.myprofile = data.member
    },
    updateDetail(state, data) {
      state.detail = data || {}
      state.myprofile = (state.myprofile || []).concat(data)
    },
    updateMine (state, data) {
      state.myprofile = data || {}
    },
    updateList (state, data) {
      const news = (data || []).filter(x => !(state.list || []).filter(y => y.MemberId === x.MemberId)[0])
      state.list = (state.list || []).concat(news)
    },
    updateTopList (state, data) {
      const news = (data || []).filter(x => !(state.toplist || []).filter(y => y.MemberId === x.MemberId)[0])
      state.toplist = (state.toplist || []).concat(news)
    },
    resetList(state, data) {
      state.list = data || []
    },
    resetTopList(state, data) {
      state.toplist = data || []
    },
    voteSuccess(state, data) {
      state.toplist = state.toplist.map(x => {
        return x.MemberId === data.MemberId ? data : x
      })
      state.list = state.list.map(x => {
        return x.MemberId === data.MemberId ? data : x
      })
    },
    resetDetail(state, memberId) {
      let mem = state.list.filter(x => x.MemberId === memberId)[0]
      if (!mem) {
        mem = state.toplist.filter(x => x.MemberId === memberId)[0]
      }

      state.detail = mem
    },
    setCode(state, code) {
      state.code = code
    }
  },
  actions: {
    getActivity ({ commit, state }, {actid}) {
      if (state.activity && state.activity.ActivityId == actid) {
        return Promise.resolve(state.activity)
      }

      return new Promise((resolve, reject) => {
        ajax({
          ...api.voteing.info,
          urlData: {
            actid: actid,
          }
        }).then((res) => {
          commit('updateActivity', res)
          resolve(res)
        }).catch(reject)
      })
    },
    getMine ({ commit, state }, payload) {
      if (state.myprofile && state.myprofile.MemberId) {
        return Promise.resolve(state.myprofile)
      }

      return new Promise((resolve, reject) => {
        ajax({
          ...api.voteing.mime,
          queryData: {
            openid: payload.openid,
          },
          urlData: {
            actid: payload.actid,
          },
        }).then((res) => {
          commit('updateMine', res)
          resolve(res)
        }).catch(reject)
      })
    },
    getMember ({ commit }, payload) {
      return new Promise((resolve, reject) => {
        ajax({
          ...api.voteing.detail,
          urlData: {
            actid: payload.actid,
            memberid: payload.memberid,
          },
        }).then((res) => {
          commit('updateDetail', res)
          resolve(res)
        }).catch(reject)
      })
    },
    vote({ commit }, payload) {
      return new Promise((resolve, reject) => {
        ajax({
          ...api.voteing.vote,
          urlData: {
            actid: payload.actid,
            memberid: payload.memberid,
          },
          queryData: {
            openid: payload.openid,
          }
        }).then((res) => {
          commit('voteSuccess', res)

          resolve(res)
        }).catch(reject)
      })
    },
    voteMember({ commit }, payload) {
      return new Promise((resolve, reject) => {
        ajax({
          ...api.voteing.registe,
          urlData: {
            actid: payload.actid,
          },
          queryData: {
            ...payload,
          }
        }).then((res) => {
          commit('updateDetail', res)
          resolve(res)
        }).catch(reject)
      })
    },
    getUser({ commit }, { code, actid }) {
      return new Promise((resolve, reject) => {
        ajax({
          ...api.voteing.user,
          queryData: {
            actid,
          },
          urlData: {
            code: code,
          },
        }).then((res) => {
          commit('updateUser', res)
          resolve(res)
        }).catch(reject)
      })
    },
    getList({ commit }, payload) {
      return new Promise((resolve, reject) => {
        ajax({
          ...api.voteing.list,
          queryData: {
            ...payload,
            tab: 1,
          },
        }).then((res) => {
          if (payload.page <= 1) {
            commit('resetList', res)
          } else {
            commit('updateList', res)
          }

          resolve(res)
        }).catch(reject)
      })
    },
    getTopList({ commit }, payload) {
      return new Promise((resolve, reject) => {
        ajax({
          ...api.voteing.list,
          queryData: {
            ...payload,
            tab: 2,
          },
        }).then((res) => {
          if (payload.page <= 1) {
            commit('resetTopList', res)
          } else {
            commit('updateTopList', res)
          }
          resolve(res)
        }).catch(reject)
      })
    },
    setCode({ commit }, code) {
      commit('setCode', code)
    },
  }
}
